# -*- coding: utf-8 -*-
# Copyright (c) 2011-2015 Polyconseil SAS. All rights reserved.

from __future__ import unicode_literals

import os
import shutil
try:  # Python 2.6 compatibility
    from unittest2 import TestCase
except ImportError:
    from unittest import TestCase

from pyramid import testing

from dokang import api
from dokang.harvesters import html_config
from dokang import views


def get_data_path(*components):
    return os.path.join(
        os.path.abspath(os.path.dirname(__file__)),
        'data',
        *components)


INDEX_PATH = get_data_path('whoosh_test_index')
TEST_DOC_SETS = (
    # Minimal configuration with just what we need to test the web
    # frontend.
    {'test': {
      'title': "Title of the test doc set"}
    }
)


class TestWebFrontEnd(TestCase):

    @classmethod
    def setUpClass(cls):
        super(TestWebFrontEnd, cls).setUpClass()
        cls._prepare_index()

    @classmethod
    def tearDownClass(cls):
        super(TestWebFrontEnd, cls).tearDownClass()
        shutil.rmtree(INDEX_PATH)

    def setUp(self):
        self.config = testing.setUp()
        self.config.include('pyramid_chameleon')
        self.config.registry.settings['dokang.doc_sets'] = TEST_DOC_SETS
        self.config.registry.settings['dokang.index_path'] = INDEX_PATH

    def tearDown(self):
        testing.tearDown()

    @classmethod
    def _prepare_index(cls):
        api.initialize_index(INDEX_PATH)
        doc_set_info = {
            'id': 'test',
            'title': 'Test documentation',
            'path': get_data_path('api'),
            'url': 'http://docs.exemple.com/',
            'harvester': html_config()
        }
        api.index_document_set(INDEX_PATH, doc_set_info)

    def test_search(self):
        request = testing.DummyRequest()
        context = views.search(request)
        self.assertEqual(context['hits'], None)

        request = testing.DummyRequest(params={
            'query': 'ShouldBeIndexed',
            'doc_set': 'not-the-right-docset'})
        context = views.search(request)
        self.assertEqual(context['hits'], [])

        request = testing.DummyRequest(params={
            'query': 'ShouldBeIndexed'})
        context = views.search(request)
        hits = context['hits']
        self.assertEqual(len(hits), 1)
        self.assertEqual(hits[0]['doc_set_title'], "Title of the test doc set")



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2015 Polyconseil SAS. All rights reserved.
from __future__ import unicode_literals

import os
import os.path

from dokang import api


def get_harvester(fqn):
    module_fqn, function_fqn = fqn.rsplit(b'.', 1)
    module = __import__(module_fqn, fromlist=[function_fqn])
    return getattr(module, function_fqn)


def doc_set(settings, uploaded):

    harvester = get_harvester(settings['dokang.uploaded_docs.harvester'])
    upload_dir = settings.get('dokang.uploaded_docs.dir')

    uploaded = uploaded.decode('utf-8')
    uploaded_path = os.path.join(upload_dir, uploaded)
    return {
        'id': uploaded,
        'title': uploaded,
        'path': uploaded_path,
        'harvester': harvester(),
    }


def load_doc_sets(settings):
    """
    Given a settings dictionary with the path of the doc sets file,
    replace the path (in-place) with the doc sets themselves.
    """
    index_path = settings['dokang.index_path']
    if not os.path.exists(index_path):
        api.initialize_index(index_path)

    upload_dir = settings['dokang.uploaded_docs.dir']
    if not os.path.exists(upload_dir):
        os.makedirs(upload_dir)

    sets = settings['dokang.doc_sets'] = {}
    for uploaded in os.listdir(upload_dir):
        sets[uploaded] = doc_set(settings, uploaded)

    return settings



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2015 Polyconseil SAS. All rights reserved.

from __future__ import unicode_literals

import os
import shutil
try:  # Python 2.6 compatibility
    from unittest2 import TestCase
except ImportError:
    from unittest import TestCase

from dokang import api
from dokang.harvesters import html_config


def get_data_path(*components):
    return os.path.join(
        os.path.abspath(os.path.dirname(__file__)),
        'data',
        *components)


class TestApi(TestCase):

    def tearDown(self):
        super(TestApi, self).tearDown()
        if os.path.exists(self.index_path):
            shutil.rmtree(self.index_path)

    @property
    def index_path(self):
        return get_data_path('whoosh_test_index')

    def test_integration(self):
        api.initialize_index(self.index_path)

        doc_set_info = {
            'id': 'test',
            'title': 'Test documentation',
            'path': get_data_path('api'),
            'url': 'http://docs.exemple.com/',
            'harvester': html_config()
        }
        api.index_document_set(self.index_path, doc_set_info)

        self.assertEqual(list(api.search(self.index_path, "ShouldNotBeIndexed")), [])
        hits = list(api.search(self.index_path, "ShouldBeIndexed"))
        self.assertEqual(len(hits), 1)
        self.assertEqual(hits[0]['path'], 'test1.html')
        self.assertEqual(hits[0]['title'], 'The title')
        self.assertEqual(hits[0]['set'], 'test')
        self.assertEqual(hits[0]['kind'], 'HTML')
        hits = list(api.search(self.index_path, "ShouldBeIndexed set:test"))
        self.assertEqual(len(hits), 1)
        self.assertEqual(list(api.search(self.index_path, "ShouldBeIndexed set:unknown")), [])

        api.clear_document_set(self.index_path, 'unknown')
        hits = list(api.search(self.index_path, "ShouldBeIndexed"))
        self.assertEqual(len(hits), 1)

        api.clear_document_set(self.index_path, doc_set_info['id'])
        self.assertEqual(list(api.search(self.index_path, "ShouldBeIndexed")), [])



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2015 Polyconseil SAS. All rights reserved.

import abc


class Harvester(object):
    """An abstract class for all harvesters."""

    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def harvest_file(self, path):
        """Harvest contents from the given file.

        This method must be implemented by subclasses. It should
        return None if the document is not to be indexed, or a
        dictionary that has the following keys. Each text-like value
        should be a string (in Python 3) or a unicode object (in
        Python 2).

        title
            The title of the document.

        content
            The concatenated content of the document.

        kind
            The kind of document: HTML, PDF, etc.
        """
        return None



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2015 Polyconseil SAS. All rights reserved.

import logging
import os
import re

from .base import Harvester  # pylint: disable=unused-import
from .html import html_config, HtmlHarvester  # pylint: disable=unused-import
from .sphinx import (  # pylint: disable=unused-import
    sphinx_config, sphinx_rtd_config,
    SphinxHarvester, ReadTheDocsSphinxHarvester
)


logger = logging.getLogger(__name__)


def _must_process_path(path, include, exclude):
    for exp in include:
        if exp.match(path):
            return True
    for exp in exclude:
        if exp.match(path):
            return False
    return True


def harvest_set(base_dir, doc_set, config, mtimes, force):
    """Harvest a document set and return documents as dictionaries.

    ``config`` is the harvester configuration. It should contain a key
    for each supported file extensions. ``mtimes`` is a dictionary
    that links the path of each indexed file to its last modification
    time. ``force`` indicates whether to reindex a document even if it
    has not ben modified since the last indexation.

    This function is a generator. It yields dictionaries. Each
    dictionary should represent a document and contain the following
    keys in addition to the keys returned by the harvester itself.
    Each text-like value should be a string (in Python 3) or a unicode
    object (in Python 2).

    path
        The path of the document relative to the root of the document
        set.
    set
        The id of the document set. It should be ``doc_set``.
    """
    config_copy = config.copy()
    include = [re.compile(exp) for exp in config_copy.pop('include') or ()]
    exclude = [re.compile(exp) for exp in config_copy.pop('exclude') or ()]
    extensions = config_copy
    for dir_path, _dir_names, file_names in os.walk(base_dir):
        for filename in file_names:
            path = os.path.join(dir_path, filename)
            relative_path = os.path.relpath(path, base_dir)
            if not _must_process_path(relative_path, include, exclude):
                logger.debug('Excluded file "%s": include/exclude rules.', relative_path)
                continue
            _, extension = os.path.splitext(filename)
            extension = extension.lstrip('.')  # remove leading dot
            harvester_class = extensions.get(extension)
            if harvester_class is None:
                logger.debug('Excluded file "%s": no harvester found for %s.', relative_path, extension)
                continue
            modification_time = os.path.getmtime(path)
            indexed_modification_time = mtimes.get(relative_path)
            if not force and (indexed_modification_time and indexed_modification_time >= modification_time):
                logger.debug('Excluded file: "%s": not modified since last indexation.', relative_path)
                continue
            try:
                logger.debug('Indexing file "%s"', relative_path)
                doc = harvester_class().harvest_file(path)
            except Exception:  # pylint: disable=broad-except
                logger.exception("Could not index document %s", path)
            else:
                if doc:
                    doc['path'] = relative_path
                    doc['set'] = doc_set
                    doc['mtime'] = modification_time
                    yield doc



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2015 Polyconseil SAS. All rights reserved.

from __future__ import unicode_literals

import os
try:  # Python 2.6 compatibility
    from unittest2 import TestCase
except ImportError:
    from unittest import TestCase

from dokang import harvesters


def get_data_path(*components):
    return os.path.join(
        os.path.abspath(os.path.dirname(__file__)),
        'data',
        *components)


class TestHtmlHarvester(TestCase):

    def test_basics(self):
        path = get_data_path('harvesters', 'html.html')
        harvester = harvesters.HtmlHarvester()
        document = harvester.harvest_file(path)
        self.assertEqual(document['title'], "The title")
        self.assertIn("ShouldBeIndexed", document['content'])
        self.assertNotIn("ShouldNotBeIndexed", document['content'])


class TestSphinxHarvester(TestCase):

    def test_basics(self):
        path = get_data_path('harvesters', 'sphinx.html')
        harvester = harvesters.SphinxHarvester()
        document = harvester.harvest_file(path)
        self.assertEqual(document['title'], "The title")
        self.assertIn("ShouldBeIndexed", document['content'])
        self.assertNotIn("ShouldNotBeIndexed", document['content'])


class TestReadTheDocsSphinxHarvester(TestCase):

    def test_basics(self):
        path = get_data_path('harvesters', 'sphinx_rtd.html')
        harvester = harvesters.ReadTheDocsSphinxHarvester()
        document = harvester.harvest_file(path)
        self.assertEqual(document['title'], "The title")
        self.assertIn("ShouldBeIndexed", document['content'])
        self.assertNotIn("ShouldNotBeIndexed", document['content'])



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2015 Polyconseil SAS. All rights reserved.

from __future__ import unicode_literals

import argparse
import logging
import logging.config
import os
import sys

from dokang import api
from dokang.utils import load_doc_sets
from dokang.version import VERSION
from dokang import compat

logger = logging.getLogger(__name__)


def main():
    kwargs = vars(parse_args(sys.argv[1:]))
    callback = kwargs.pop('callback')
    settings = kwargs.pop('settings') or os.environ.get('DOKANG_SETTINGS')
    if not settings:
        sys.exit("You must provide the settings location.")
    kwargs['settings'] = load_settings(settings)
    return callback(**kwargs)


def init(settings, force):
    index_path = settings['dokang.index_path']
    if os.path.exists(index_path) and not force:
        sys.exit("Index already exists at %s. Use `--force` to overwrite it." % index_path)
    api.initialize_index(index_path)


def index(settings, only_doc_set, force):
    index_path = settings['dokang.index_path']
    doc_sets = settings['dokang.doc_sets']
    for doc_set, info in doc_sets.items():
        if only_doc_set is not None and only_doc_set != doc_set:
            continue
        api.index_document_set(index_path, info, force)


def clear(settings, docset):
    index_path = settings['dokang.index_path']
    api.clear_document_set(index_path, docset)


def search(settings, query):
    if isinstance(query, bytes):
        query = query.decode(sys.stdin.encoding)
    index_path = settings['dokang.index_path']
    hits = list(api.search(index_path, query, limit=None))
    compat.print_to_stdout("Found %d results." % len(hits))
    for hit in hits:
        compat.print_to_stdout("[{set}] {title}".format(**hit))


def parse_args(args):
    parser = argparse.ArgumentParser(
        prog="Dokang",
        description='A lightweight search engine for HTML documents.'
    )
    parser.add_argument('--version', action='version', version='%%(prog)s %s' % VERSION)
    parser.add_argument('--settings', action='store', help="Path of the setting file.", metavar='PATH')

    subparsers = parser.add_subparsers()

    # init
    parser_init = subparsers.add_parser('init', help='Initialize the index.')
    parser_init.set_defaults(callback=init)
    parser_init.add_argument(
        '--force',
        action='store_true',
        help="If set, delete the index if it already exists."
    )

    # index
    parser_index = subparsers.add_parser('index', help='Populate the index.')
    parser_index.set_defaults(callback=index)
    parser_index.add_argument(
        '--docset',
        action='store',
        dest='only_doc_set',
        metavar='DOC_SET_ID',
        help="If set, only index the given document set."
    )
    parser_index.add_argument(
        '--force',
        action='store_true',
        help="If set, reindex all documents even those that have not been modified since the last indexation."
    )

    # clear
    parser_clear = subparsers.add_parser('clear', help='Remove a document set.')
    parser_clear.set_defaults(callback=clear)
    parser_clear.add_argument(
        'docset',
        action='store',
        metavar='DOC_SET_ID',
        help="The id of the document set to remove."
    )

    # search
    parser_search = subparsers.add_parser('search', help="Search in the index.")
    parser_search.set_defaults(callback=search)
    parser_search.add_argument('query', metavar='QUERY')

    return parser.parse_args(args)


def load_settings(path):
    here = os.path.abspath(os.path.dirname(path))
    config = compat.ConfigParser(defaults={'here': here})
    config.read(path)
    settings = dict(config.items('app:main'))
    logging.config.fileConfig(path)
    settings = load_doc_sets(settings)
    return settings


if __name__ == '__main__':
    main()



import os
import re
import sys

from setuptools import setup, find_packages

# Do not try to import the package to get its version.
_version_file = open(os.path.join(os.path.dirname(__file__), 'dokang', 'version.py'))
VERSION = re.compile(r"^VERSION = '(.*?)'", re.S).match(_version_file.read()).group(1)


def load_requirements(path, dev=True):
    reqs = []
    with open(path) as fp:
        reqs = [line for line in fp.read().split("\n")
            if line and not line.startswith(("-r", "#"))]
    return reqs


here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README')).read().strip()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read().strip()

setup(name='Dokang',
      version=VERSION,
      description="Lightweight search engine with a web frontend",
      long_description=README + '\n\n' + CHANGES,
      classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Framework :: Pyramid",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
        "Topic :: Text Processing :: Markup :: HTML",
        ],
      author="Polyconseil",
      author_email="opensource+dokang@polyconseil.fr",
      url='',
      keywords='full-text search engine',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires=load_requirements('requirements.txt'),
      tests_require=load_requirements('requirements_dev.txt', dev=True),
      test_suite='tests',
      entry_points="""\
      [paste.app_factory]
      main = dokang.app:make_app

      [console_scripts]
      dokang = dokang.cli:main
      """,
      )
